#! /usr/bin/python

import threading

class DynamicBarrier(object):
    def __init__(self):
        self._lock = threading.Lock()
        self._cond = threading.Condition()

    def enter(self):
        """
        Enter the dynamic barrier. Returns True if you should be
        the one performing the operation. False if someone already
        did that for you.

        You only have to exit() if you actually entered.

        Example:

        >> if dynamicBarrier.enter():
        >>    print "Do stuff"
        >>    dynamicBarrier.exit()
        """

        self._cond.acquire()
        try:
            print("### lock 0.")
            if self._lock.acquire(False):
                return True
            print("### error !.cond wait 1.")
            exit()
            self._cond.wait()
            print("### cond wait 2.")
            if self._lock.acquire(False):
                return True
            print("### cond wait 3.")
            self._cond.wait()
            return False

        finally:
            self._cond.release()

    def exit(self):
        self._cond.acquire()
        try:
            self._lock.release()
            self._cond.notifyAll()
        finally:
            self._cond.release()

if __name__ == "__main__":
    i = 0
    while i < 1000:
        i = i + 1
        print "round %d" % i
        barrier = DynamicBarrier()
        barrier.enter()
        barrier.exit()

