#! /usr/bin/python

import sys
import uuid
import time
import os

sys.path.append('/usr/share/vdsm')

from vdsm import vdscli
from storage.sd import LOCALFS_DOMAIN, DATA_DOMAIN
from storage.volume import COW_FORMAT, SPARSE_VOL, LEAF_VOL, BLANK_UUID, PREALLOCATED_VOL

path = "/var/lib/vdsm/storage"
vdsm_uid = 36
kvm_gid = 36
masterVersion = 1
hostID = 1

spUUID = '5c84a9d2-be25-49a9-bcb2-2f0f5b9066c0' 
sdUUID = 'b9c4821e-e9c0-4dd4-8393-c59b82c9cd29'
imgUUID = '87f5f391-5b83-4e7f-9072-7f50266559c3'
volUUID = 'ce73fdd0-2630-40ca-b2df-a0dd90674dec'
vmId = 'f7c1b02f-b304-4e68-8565-3659b1214cba'

def vdsOK(d):
    print d
    if d['status']['code']:
        raise Exception(str(d))
    return d

def waitTask(s, taskid):
    while vdsOK(s.getTaskStatus(taskid))['taskStatus']['taskState'] != 'finished':
        print "waiting for the task %s" % taskid
        time.sleep(3)
    vdsOK(s.clearTask(taskid))

def create_delete_image_vol(s):
    print "creating volume"
    sizeGiB = 1
    #sizeGiB = 10
    sectors_per_GB = 2097152
    size = sizeGiB * sectors_per_GB

    tid = vdsOK(s.createVolume(sdUUID, spUUID, imgUUID, size,
                       COW_FORMAT, SPARSE_VOL, LEAF_VOL,
                       volUUID, "volly",
                       BLANK_UUID, BLANK_UUID))['uuid']
    waitTask(s, tid)

    print "deleting image"
    tid = vdsOK(s.deleteImage(sdUUID, spUUID, imgUUID))['uuid']
    waitTask(s, tid)

def create_delete_sp(s):
    print "creating sp"
    vdsOK(s.createStoragePool(LOCALFS_DOMAIN, spUUID, "pool name", sdUUID, [sdUUID], masterVersion))

    print "connecting sp"
    vdsOK(s.connectStoragePool(spUUID, hostID, "scsikey", sdUUID, masterVersion))
    vdsOK(s.getConnectedStoragePoolsList())

    print "becoming SPM"
    tid = vdsOK(s.spmStart(spUUID, -1, -1, -1, 0))['uuid']
    waitTask(s, tid)

    print "destroy sp"
    vdsOK(s.destroyStoragePool(spUUID, hostID, "scsikey"))
    vdsOK(s.getConnectedStoragePoolsList())

def create_delete_sd(s):
    if not os.access(path, os.W_OK):
        os.mkdir(path)
        os.chown(path, vdsm_uid, kvm_gid)
    print "connecting LOCALFS"
    vdsOK(s.connectStorageServer(LOCALFS_DOMAIN, "my favorite pet", [dict(id=1, connection=path)]))

    print "creating sd"
    vdsOK(s.createStorageDomain(LOCALFS_DOMAIN, sdUUID, "my local domain", path, DATA_DOMAIN, 0))
    print "show result"
    vdsOK(s.getStorageDomainsList())

    #print "detaching sd"
    #vdsOK(s.detachStorageDomain(sdUUID, spUUID, BLANK_UUID, masterVersion))

    #print "deactivating sd"
    #vdsOK(s.deactivateStorageDomain(sdUUID, spUUID, BLANK_UUID, masterVersion))

    #print "releasing SPM"
    #vdsOK(s.spmStop(spUUID))

    #print "disconnecting sp"
    #vdsOK(s.disconnectStoragePool(spUUID, hostID, "scsikey"))

    print "formating sd"
    vdsOK(s.formatStorageDomain(sdUUID))
    print "show result"
    vdsOK(s.getStorageDomainsList())

    print "disconnecting LOCALFS"
    vdsOK(s.disconnectStorageServer(LOCALFS_DOMAIN, BLANK_UUID, [dict(id=1, connection=path)]))

def main():
    print "start test."
    s = vdscli.connect()
    i = 0
    while i < 1:
        i = i + 1
        print "round %d" % i
        create_delete_sd(s)

if __name__ == "__main__":
    main()
